from django.db import models
from django.db.models.functions import Coalesce

from wagtail.core.fields import StreamField
from wagtail.search import index

from modelcluster.contrib.taggit import ClusterTaggableManager
from modelcluster.fields import ParentalKey
from taggit.models import TaggedItemBase

from pfmsite.core.blocks import BodyBlock
from pfmsite.utils.pages import AbstractBasePage


class BlogPage(AbstractBasePage):
    """Une page d'index d'actualités"""

    class Meta:
        verbose_name = "index d'actualités"

    subpage_types = ['blog.ArticlePage']

    def get_context(self, request):
        context = super().get_context(request)
        context['articles'] = self.get_articles()
        return context

    def get_articles(self):
        return (
            ArticlePage.objects.descendant_of(self)
            .live()
            .order_by(Coalesce('go_live_at', 'first_published_at').desc())
        )


class ArticlePage(AbstractBasePage):
    """Un article d'actualité"""

    tags = ClusterTaggableManager(
        through='ArticlePageTag',
        blank=True,
        verbose_name="étiquettes",
    )
    introduction = models.TextField(
        max_length=128,
        help_text="Texte court introductif qui résume l'actualité",
    )
    image = models.ForeignKey(
        'wagtailimages.Image',
        verbose_name="illustation",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
    )
    body = StreamField(BodyBlock(), verbose_name="contenu")

    class Meta:
        verbose_name = "article"
        verbose_name_plural = "articles"

    parent_page_types = ['blog.BlogPage']
    subpage_types = []
    search_fields = AbstractBasePage.search_fields + [
        index.SearchField('body', partial_match=True),
    ]

    @property
    def date(self):
        return self.go_live_at or self.first_published_at


class ArticlePageTag(TaggedItemBase):
    """
    Relation intermédiaire entre un article et une étiquette.
    """

    content_object = ParentalKey(
        'ArticlePage',
        on_delete=models.CASCADE,
        related_name='article_tags',
    )
