from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.functions import Coalesce
from django.utils.safestring import mark_safe

from wagtail.contrib.settings.models import BaseSetting, register_setting
from wagtail.core.fields import RichTextField, StreamField
from wagtail.search import index

from wagtail_cblocks.blocks import ButtonBlock, Style
from wagtail_webradio.blocks import PodcastTagChooserBlock
from wagtail_webradio.models import Podcast

from pfmsite.blog.models import ArticlePage
from pfmsite.utils.pages import AbstractBasePage

from .blocks import BodyBlock, MenuItemBlock


def get_podcast_or_article_date(obj):
    """retrouve la date d'un `Podcast` ou un `ArticlePage`"""

    if isinstance(obj, ArticlePage):
        return obj.date
    elif isinstance(obj, Podcast):
        return obj.publish_date


class StandardPage(AbstractBasePage):
    class Meta:
        verbose_name = "page standard"
        verbose_name_plural = "pages standards"

    body = StreamField(BodyBlock, blank=True, verbose_name="contenu de la page")

    search_fields = AbstractBasePage.search_fields + [
        index.SearchField('body', partial_match=True)
    ]


class HomePage(AbstractBasePage):
    class Meta:
        verbose_name = "page d'accueil"

    tags = StreamField(
        [
            ('tag', PodcastTagChooserBlock()),
        ],
        max_num=3,
    )

    # uniquement à la racine
    parent_page_types = ['wagtailcore.Page']

    def get_context(self, request, *args, **kwargs):
        """
        On ajoute un dictionnaire agrégeant `Podcast` et `ArticlePage` tagués
        """

        context = super().get_context(request, *args, **kwargs)

        tagged_contents = {}
        tags = [tag.value for tag in self.tags if tag.value]

        # Deux querysets séparées car les models sont trop différents pour une
        # union
        podcasts_qs = (
            Podcast.objects.currents()
            .filter(tags__name__in=[*tags])
            .order_by('-publish_date')[:15]
        )
        actus_qs = ArticlePage.objects.filter(tags__name__in=[*tags]).order_by(
            Coalesce('go_live_at', 'first_published_at').desc()
        )[:15]

        # jointure en python
        mixed_items = [*podcasts_qs, *actus_qs]
        for tag in tags:
            # On peuple avec les items selon leur tag
            tagged_contents[tag] = [
                item for item in mixed_items if tag in item.tags.all()
            ]
            # On les tri par date et on limite à 5 items max par colonne
            tagged_contents[tag].sort(
                key=get_podcast_or_article_date,
                reverse=True,
            )
            tagged_contents[tag] = tagged_contents[tag][:5]
            # On retire ces items de la liste afin d'éviter les doublons
            mixed_items = [
                item for item in mixed_items if item not in tagged_contents[tag]
            ]
        context['tagged_contents'] = tagged_contents
        return context


@register_setting
class MenuSettings(BaseSetting):
    class Meta:
        verbose_name = 'menu pricipal'

    menu = StreamField(
        [
            ('item', MenuItemBlock(label="Élément de menu")),
        ],
        verbose_name="Éléments de menu",
    )


@register_setting(icon="view")
class DisplaySettings(BaseSetting):
    class Meta:
        verbose_name = 'Apparence'

    # Header
    header_banner = models.ForeignKey(
        'wagtailimages.Image',
        verbose_name="Bannière",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
        help_text=(
            "Une bannière personnalisée qui doit être raccordable "
            "horizontalement. Elle doit faire 235px de haut, sinon, elle sera"
            "automatiquement redimensionnée"
        ),
    )
    TEXT_COLOR_CHOICES = [
        ('--bs-body-color', "défaut"),
        ('--bs-primary', "rouge"),
        ('--bs-white', "blanc"),
    ]
    banner_text_color = models.CharField(
        max_length=24,
        choices=TEXT_COLOR_CHOICES,
        default='--bs-body-color',
        verbose_name="Couleur du texte",
        help_text=(
            "Si vous changez de bannière, vous pouvez être amené à vouloir"
            "changer la couleur du texte"
        ),
    )

    # Infos pratiques
    infos = RichTextField(
        blank=True,
        verbose_name="colone de gauche",
        features=['bold', 'italic', 'link'],
    )
    email = models.EmailField(blank=True, verbose_name="adresse mail")
    phone = models.CharField(
        blank=True, max_length=32, verbose_name="Numéro de téléphone"
    )
    legal_page = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name='Page des mentions légales',
    )
    buttons = StreamField(
        [
            (
                'bouton',
                ButtonBlock(
                    styles=[
                        Style('success', 'Vert', 'btn-success'),
                        Style('info', 'Bleu', 'btn-info'),
                    ],
                    default_style='success',
                ),
            )
        ],
        blank=True,
        verbose_name="Boutons du pied de page",
    )

    # Images par défaut
    radioshow_default_picture = models.ForeignKey(
        'wagtailimages.Image',
        verbose_name="Image par défaut des émissions",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )

    podcast_default_picture = models.ForeignKey(
        'wagtailimages.Image',
        verbose_name="Image par défaut des podcasts",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )

    tracking_code = models.TextField(
        "code de suivi",
        blank=True,
        help_text=mark_safe(
            "Le code JavaScript à exécuter pour suivre les visiteurs sur le "
            "site, sans la balise <code>&lt;script&gt;</code>."
        ),
    )

    def clean(self):
        if self.tracking_code and self.tracking_code.strip()[0] == '<':
            raise ValidationError(
                {
                    'tracking_code': (
                        "Le code JavaScript semble invalide, il ne devrait "
                        "pas commencer par « < »."
                    )
                }
            )
